FFmpeg
Loading...
Searching...
No Matches
softfloat.c
Go to the documentation of this file.
1/*
2 * copyright (c) 2006 Michael Niedermayer <michaelni@gmx.at>
3 *
4 * This file is part of FFmpeg.
5 *
6 * FFmpeg is free software; you can redistribute it and/or
7 * modify it under the terms of the GNU Lesser General Public
8 * License as published by the Free Software Foundation; either
9 * version 2.1 of the License, or (at your option) any later version.
10 *
11 * FFmpeg is distributed in the hope that it will be useful,
12 * but WITHOUT ANY WARRANTY; without even the implied warranty of
13 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
14 * Lesser General Public License for more details.
15 *
16 * You should have received a copy of the GNU Lesser General Public
17 * License along with FFmpeg; if not, write to the Free Software
18 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
19 */
20
21#include "libavutil/timer.h"
22
23#include <inttypes.h>
24
25#include "libavutil/softfloat.h"
26#include "libavutil/common.h"
27#include "libavutil/log.h"
28
29#include <stdio.h>
30#include <string.h>
31
32static const SoftFloat FLOAT_0_017776489257 = {0x1234, 12};
33static const SoftFloat FLOAT_1374_40625 = {0xabcd, 25};
34static const SoftFloat FLOAT_0_1249694824218 = {0xFFF, 15};
35
36static double add_mul(int i)
37{
38 double d1 = i, d2 = i/100.0;
39
40 for (int j = 0; j < 1000; j++)
41 d1 = (d1+1)*d2;
42 return d1;
43}
44
46{
47 SoftFloat one = av_int2sf(1, 0);
48 SoftFloat sf1 = av_int2sf(i, 0);
49 SoftFloat sf2 = av_div_sf(av_int2sf(i, 2), av_int2sf(200, 3));
50
51 for (int j = 0; j < 1000; j++)
52 sf1 = av_mul_sf(av_add_sf(sf1, one), sf2);
53 return sf1;
54}
55
56int main(int argc, char **argv){
57 SoftFloat one= av_int2sf(1, 0);
58 SoftFloat sf1, sf2, sf3;
59 double d1, d2, d3;
60 int i, j;
61 int bench = argc > 1 && !strcmp(argv[1], "-t");
63
64 d1= 1;
65 for(i= 0; i<10; i++){
66 d1= 1/(d1+1);
67 }
68 printf("test1 double=%d\n", (int)(d1 * (1<<24)));
69
70 sf1= one;
71 for(i= 0; i<10; i++){
72 sf1= av_div_sf(one, av_normalize_sf(av_add_sf(one, sf1)));
73 }
74 printf("test1 sf =%d\n", av_sf2int(sf1, 24));
75
76
77 if (bench) {
78 for(i= 0; i<100; i++){
80 d1= add_mul(i);
81 STOP_TIMER("float add mul")
82 }
83 } else
84 d1= add_mul(99);
85 printf("test2 double=%d\n", (int)(d1 * (1<<24)));
86
87 if (bench) {
88 for(i= 0; i<100; i++){
90 sf1= add_mul_sf(i);
91 STOP_TIMER("softfloat add mul")
92 }
93 } else
94 sf1= add_mul_sf(99);
95 printf("test2 sf =%d (%d %d)\n", av_sf2int(sf1, 24), sf1.exp, sf1.mant);
96
97 d1 = 0.0177764893;
98 d2 = 1374.40625;
99 d3 = 0.1249694824;
100 d2 += d1;
101 d3 += d2;
102 printf("test3 double: %.10lf\n", d3);
103
105 sf2 = FLOAT_1374_40625;
107 sf2 = av_add_sf(sf1, sf2);
108 sf3 = av_add_sf(sf3, sf2);
109 printf("test3 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf3), sf3.mant, sf3.exp);
110
111 sf1 = av_int2sf(0xFFFFFFF0, 0);
112 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
113 sf1 = av_int2sf(0x00000010, 0);
114 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
115
116 sf1 = av_int2sf(0x1FFFFFFF, 0);
117 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
118 sf1 = av_int2sf(0xE0000001, 0);
119 printf("test4 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
120
121
122 sf1 = (SoftFloat){ 0x20000000, MIN_EXP };
123 sf1 = av_mul_sf(sf1, sf1);
124 printf("test5 softfloat: %.10lf (0x%08x %d)\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp);
125
126 sf1 = (SoftFloat){ 0x20000000, MIN_EXP };
127 sf2 = (SoftFloat){ 0x20000000, MAX_EXP };
128 i = av_cmp_sf(sf1, sf2);
129 j = av_cmp_sf(sf2, sf1);
130 sf1 = av_div_sf(sf1, sf2);
131 printf("test6 softfloat: %.10lf (0x%08x %d) %d %d\n", (double)av_sf2double(sf1), sf1.mant, sf1.exp, i, j);
132
133 for(i= -50; i<50; i++) {
134 sf1= av_int2sf(i, 0);
135 for(j= -50; j<50; j++) {
136 int c;
137 sf2= av_int2sf(j, 0);
138 c = av_cmp_sf(sf1, sf2);
139 if (FFDIFFSIGN(i,j) != c && (FFDIFFSIGN(i,j)^c)<0) {
140 printf("av_cmp_sf failed at %d %d as %X\n", i, j, c);
141 }
142 c = av_gt_sf(sf1, sf2);
143 if ((i>j) != c) {
144 printf("av_gt_sf failed at %d %d as %X\n", i, j, c);
145 }
146 }
147 sf1 = av_int2sf(1, i);
148 for(j = -50; j < 50; j++) {
149 int c;
150 sf2 = av_int2sf(1, j);
151 c = av_cmp_sf(sf2, sf1);
152 if (FFDIFFSIGN(i,j) != c && (FFDIFFSIGN(i,j)^c) < 0) {
153 printf("av_cmp_sf failed2 at %d %d as %X\n", i, j, c);
154 }
155 c = av_gt_sf(sf1, sf2);
156 if ((i<j) != c) {
157 printf("av_gt_sf failed2 at %d %d as %X\n", i, j, c);
158 }
159 }
160 }
161
162
163 for(i= 0; i<4*36; i++){
164 int s, c;
165 double errs, errc;
166
167 av_sincos_sf(i*(1ULL<<32)/36/4, &s, &c);
168 errs = (double)s/ (1<<30) - sin(i*M_PI/36);
169 errc = (double)c/ (1<<30) - cos(i*M_PI/36);
170 if (fabs(errs) > 0.000000004 || fabs(errc) >0.000000004) {
171 printf("sincos FAIL %d %f %f %f %f\n", i, (float)s/ (1<<30), (float)c/ (1<<30), sin(i*M_PI/36), cos(i*M_PI/36));
172 }
173
174 }
175 return 0;
176
177}
#define i(width, name, range_min, range_max)
Definition cbs_h264.c:63
#define s(width, name)
Definition cbs_vp9.c:198
#define bench(...)
Definition checkasm.h:139
common internal and external API header
__device__ int printf(const char *,...)
static __device__ float fabs(float a)
int main
Definition dovi_rpuenc.c:38
#define AV_LOG_DEBUG
Stuff which is only useful for libav* developers.
Definition log.h:231
void av_log_set_level(int level)
Set the log level.
Definition log.c:476
#define FFDIFFSIGN(x, y)
Comparator.
Definition macros.h:45
#define M_PI
Definition mathematics.h:67
static double add_mul(int i)
Definition softfloat.c:36
static const SoftFloat FLOAT_0_1249694824218
Definition softfloat.c:34
static const SoftFloat FLOAT_0_017776489257
Definition softfloat.c:32
static const SoftFloat FLOAT_1374_40625
Definition softfloat.c:33
static SoftFloat add_mul_sf(int i)
Definition softfloat.c:45
static av_const SoftFloat av_add_sf(SoftFloat a, SoftFloat b)
Definition softfloat.h:162
static av_const int av_cmp_sf(SoftFloat a, SoftFloat b)
Compares two SoftFloats.
Definition softfloat.h:138
static av_const int av_sf2int(SoftFloat v, int frac_bits)
Converts a SoftFloat to an integer.
Definition softfloat.h:198
static av_const SoftFloat av_div_sf(SoftFloat a, SoftFloat b)
b has to be normalized and not zero.
Definition softfloat.h:116
static av_unused void av_sincos_sf(int a, int *s, int *c)
Rounding-to-nearest used.
Definition softfloat.h:244
#define MIN_EXP
Definition softfloat.h:30
static av_const SoftFloat av_mul_sf(SoftFloat a, SoftFloat b)
Definition softfloat.h:102
static av_const double av_sf2double(SoftFloat v)
Convert a SoftFloat to a double precision float.
Definition softfloat.h:52
static av_const SoftFloat av_int2sf(int v, int frac_bits)
Converts a mantisse and exponent to a SoftFloat.
Definition softfloat.h:185
#define MAX_EXP
Definition softfloat.h:31
static av_const SoftFloat av_normalize_sf(SoftFloat a)
Definition softfloat.h:57
static av_const int av_gt_sf(SoftFloat a, SoftFloat b)
Compares two SoftFloats.
Definition softfloat.h:150
int32_t mant
Definition softfloat.h:35
int32_t exp
Definition softfloat.h:36
high precision timer, useful to profile code
#define START_TIMER
Definition timer.h:164
#define STOP_TIMER(id)
Definition timer.h:165
static double c[64]